<?php

namespace App\Http\Controllers\Admin\Devices;

use App\Http\Controllers\Controller;
use Illuminate\Support\Facades\Schema;
use Microsoft\Graph\Graph;
use Microsoft\Graph\Model;
use Illuminate\Support\Facades\DB;

class SyncDevicesFromAzureADtoDbController extends Controller
{
    public function syncDevicesInfo()
    {
        $this->loadViewData();

        $graph = new Graph();

        $graph->setAccessToken(session('accessToken'));

        $employees = DB::table('rh_employees')
            ->select('EMPLOYEE_IDAAD', 'EMPLOYEE_ID', 'EMPLOYEE_EMAILPRO')
            ->get();

        DB::table('rh_devices')
            ->truncate();

        foreach ($employees as $employee) {

            $devicesId = '';

            try {
                $devicesId = $graph->createRequest('GET', '/users/' . $employee->EMPLOYEE_EMAILPRO . '/managedDevices?$select=id,serialNumber,azureADDeviceId,azureADRegistered,enrolledDateTime,managedDeviceName,deviceRegistrationState,model,totalStorageSpaceInBytes,freeStorageSpaceInBytes,deviceName,lastSyncDateTime,osVersion')
                    ->setReturnType(Model\ManagedDevice::class)
                    ->execute();

                $devicesId = json_decode(json_encode($devicesId[0]));
            } catch (\Exception $e) {
                // ignorer l'erreur
            }

            try {
                if ($devicesId != '') {
                    $totalStorageSpaceInGB = $devicesId->totalStorageSpaceInBytes / (1024 * 1024 * 1024);
                    $totalFreeSpaceInGB =  $devicesId->freeStorageSpaceInBytes / (1024 * 1024 * 1024);
                    $enrolledDateTime = date('Y-m-d h:m:s', strtotime($devicesId->enrolledDateTime));
                    $lastSyncDateTime = date('Y-m-d h:m:s', strtotime($devicesId->lastSyncDateTime));

                    DB::table('rh_devices')->insert([

                        'DEVICE_FKEMPLOYEEID' => $employee->EMPLOYEE_ID,
                        'DEVICE_ID' => $devicesId->id,
                        'DEVICE_AADDEVICEID' => $devicesId->azureADDeviceId,
                        'DEVICE_MODEL' => $devicesId->model,
                        'DEVICE_SERIALNUMBER' => $devicesId->serialNumber,
                        'DEVICE_NOM' => $devicesId->deviceName,
                        'DEVICE_ENROLLEDDATETIME' => $enrolledDateTime,
                        'DEVICE_LASTSYNC' => $lastSyncDateTime,
                        'DEVICE_TOTALSTORAGE' => $totalStorageSpaceInGB,
                        'DEVICE_FREESTORAGE' => $totalFreeSpaceInGB

                    ]);
                }
            } catch (\Exception $e) {
                // ignorer l'erreur
            }
        }

        return back()->withInput();
    }
}
