<?php

namespace App\Http\Controllers\Admin\Employees;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use Microsoft\Graph\Graph;
use Microsoft\Graph\Model;
use Illuminate\Support\Facades\DB;

class SyncAllUsersFromAzureAdController extends Controller
{
    public function syncAllUsers()
    {
        $this->loadViewData();

        $graph = new Graph();

        $graph->setAccessToken(session('accessToken'));

        $membersOfIcc = $graph->createRequest('GET', '/groups/' . env('ALLUSERS_GROUP_ID') . '/members?$select=id,displayName,userPrincipalName,mail,surname,givenName,accountEnabled&$top=300') // Cherche les membres du groupe "Tous les utilisateurs"
            ->setReturnType(Model\Group::class)
            ->execute();

        foreach ($membersOfIcc as $member) {
            try {
                $member = json_decode(json_encode($member));
                $userMail = $member->mail;
                $userDisplayName = $member->displayName;
                if (
                    !str_contains($userMail, 'onmicrosoft.com') &&
                    !str_contains($userMail, 'support') &&
                    !str_contains($userMail, 'bookings') &&
                    !str_contains($userMail, 'no.reply') &&
                    !str_contains($userDisplayName, '🍁') &&
                    !str_contains($userDisplayName, '📆') &&
                    !str_contains($userDisplayName, '🛸') &&
                    !str_contains($userDisplayName, '🚔') &&
                    !str_contains($userDisplayName, '🎈') &&
                    !str_contains($userDisplayName, '🏠') &&
                    !str_contains($userDisplayName, '🍀')
                ) {
                    $yetInDB = DB::table('rh_employees')
                        ->select('EMPLOYEE_EMAILPRO')
                        ->where('EMPLOYEE_EMAILPRO', '=', $userMail)
                        ->count();
                    if ($yetInDB == 0) {

                        if ($member->accountEnabled) {
                            $userAccountEnabled = $member->accountEnabled;
                        }
                        else {
                            $userAccountEnabled = 2;
                        }
                        // Ajout en base de données et récupère son Id local
                        $newEmployeeId = DB::table('rh_employees')
                            ->insertGetId([
                                'EMPLOYEE_EMAILPRO' => $userMail,
                                'EMPLOYEE_IDAAD' => $member->id,
                                'EMPLOYEE_PRENOM' => $member->givenName,
                                'EMPLOYEE_NOM' => strtoupper($member->surname),
                                'EMPLOYEE_COMPTEACTIF' => $userAccountEnabled,
                                'EMPLOYEE_FKAGENCEID' => 22 // Par défaut, agence "aucune"
                            ]);

                        // Ajout une première entrée null dans Orias pour éviter les erreurs
                        DB::table('rh_orias')->insert([
                            'ORIAS_FKEMPLOYEEID' => $newEmployeeId,
                        ]);

                        // Ajout une première entrée null dans Ieam pour éviter les erreurs
                        DB::table('rh_ieam')->insert([
                            'IEAM_FKEMPLOYEEID' => $newEmployeeId,
                        ]);

                        // Ajout une première entrée null dans Afib pour éviter les erreurs
                        DB::table('rh_afib')->insert([
                            'AFIB_FKEMPLOYEEID' => $newEmployeeId,
                        ]);
                    }
                }
            } catch (\Exception $e) {
                dd($e, $member);
                // Ignorer les erreurs
            }
        }

        return back()->withInput();
    }
}
